#include <math.h>
#include <float.h>
#include <stdbool.h>

#define PI        3.141592653589793238462643383279502884197L

#define MAX(a, b) ((a) > (b) ? (a) : (b))
#define MIN(a, b) ((a) < (b) ? (a) : (b))

double Kepler(double x, double e)
{
    int cnt = 0;
    bool neg_e = false, adjust = false;
    double x1, x2, s, c, d1, d2, d, a, b, v;
    if(fabs(e) > 1.0)
        return 0.0L;
    if(sin(x) == 0.0)
        return x;
    if(e < 0)
    {
        e = -e;
        x -= PI;
        neg_e = true;
    }
    a = x - e;
    b = x + e;
    if(sin(x / 2) > 0)
        x1 = MAX(a, (long double)(lround((x - PI) / (2 * PI))) * (2 * PI) + PI);
    else
        x1 = MIN(b, (long double)(lround((x - PI) / (2 * PI))) * (2 * PI) + PI);
    sincos(x1, &s, &c);
    v = (1 - e * c);
    d1 = (x1 - e * s - x) / v;
    do
    {
        x2 = x1;
        d2 = d1;
        x1 -= d1;

        sincos(x1 , &s, &c);
        d1 = (x1 - e * s - x);

        if(d1 == 0.0L)
            break;
        if((d1 < 0) == (d2 > 0))
        {
            ++cnt;
            if(cnt > 1)
            {
                adjust = true;
                break;
            }
        }
        if(cnt == 0)
            v = (1 - e * c);
        d1 /= v;
    }while(fabs((x1 - x2) / x2) > 2 * DBL_EPSILON);
    if(adjust)
    {
        do
        {
            c = (x1 + x2) / 2;
            d = c - e * sin(c) - x;
            if(d == 0)
            {
                x1 = c;
                break;
            }
            if((d < 0) == (d1 > 0))
            {
                x2 = c;
                d2 = d;
            }
            else
            {
                x1 = c;
                d1 = d;
            }
        }while(fabs((x1 - x2) / x2) > DBL_EPSILON);
        x1 = c;
    }
    if(neg_e) return x1 + PI;
    else return x1;
}


long double KeplerL(long double x, long double e)
{
    int cnt = 0;    
    bool neg_e = false, adjust = false;
    long double x1, x2, s, c, d1, d2, d, a, b, v;
    if(fabsl(e) > 1.0L)
        return 0.0L;
    if(sinl(x) == 0.0L)
        return x;
    if(e < 0)
    {
        e = -e;
        x -= PI;
        neg_e = true;
    }
    a = x - e;
    b = x + e;
    if(sinl(x / 2) > 0)
        x1 = MAX(a, (long double)(lroundl((x - PI) / (2 * PI))) * (2 * PI) + PI);
    else
        x1 = MIN(b, (long double)(lroundl((x - PI) / (2 * PI))) * (2 * PI) + PI);
    sincosl(x1, &s, &c);
    v = (1 - e * c);
    d1 = (x1 - e * s - x) / v;
    do
    {
        x2 = x1;
        d2 = d1;
        x1 -= d1;

        sincosl(x1 , &s, &c);
        d1 = (x1 - e * s - x);

        if(d1 == 0.0L)
            break;
        if((d1 < 0) == (d2 > 0))
        {
            ++cnt;
            if(cnt > 1)
            {
                adjust = true;
                break;
            }
        }
        if(cnt == 0)
            v = (1 - e * c);
        d1 /= v;
    }while(fabsl((x1 - x2) / x2) > 2 * LDBL_EPSILON);
    if(adjust)
    {
        do
        {
            c = (x1 + x2) / 2;
            d = c - e * sinl(c) - x;
            if(d == 0)
            {
                x1 = c;
                break;
            }
            if((d < 0) == (d1 > 0))
            {
                x2 = c;
                d2 = d;
            }
            else
            {
                x1 = c;
                d1 = d;
            }
        }while(fabsl((x1 - x2) / x2) > LDBL_EPSILON);
        x1 = c;
    }
    if(neg_e) return x1 + PI;
    else return x1;
}